!---------------------------------------------------------------------------------------
! PREP_SYNDATA: mod_var
!---------------------------------------------------------------------------------------
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!
!  Copyright 2017, Rona Thompson
!---------------------------------------------------------------------------------------
!
!> mod_var 
!!
!---------------------------------------------------------------------------------------

module mod_var

  implicit none

  ! fixed parameters 
  ! -----------------

  integer, parameter :: max_path_len=200          ! max character length of paths and files
  integer, parameter :: max_name_len=50           ! max character length of variable names
  integer, parameter :: recname_len=3             ! length of receptor names
  real, parameter    :: numscale=1.e12            ! numeric scaling factor
  integer, parameter :: maxpoint=1000             ! max number of releases per receptor per month
  integer, parameter :: maxspec=10                ! max number of species in a flexpart run
  integer, parameter :: maxlev=50                 ! max number of vertical levels in flexpart output
  integer, parameter :: maxobs=10000              ! max number of observations
  real, parameter    :: trunc=1.e-9              ! truncation threshold for eigenvalues of error covariance matrix

  ! flexpart variables 
  ! ------------------

  integer            :: ageclass                 ! length of backtrajectories (s)
  integer            :: trajdays                 ! length of backtrajectories (days)
  integer            :: ndgrid                   ! number of footprints per day
  integer            :: maxngrid                 ! max number of footprints (ndgrid*trajdays+2)
  integer            :: nxgrid                   ! number of longitudinal grid cells
  integer            :: nygrid                   ! number of latitudinal grid cells
  integer            :: nzgrid                   ! number of vertical layers in grid_init files
  integer            :: nzlev                    ! number of vertical layers in grid_time files (nzlev = 1)
  real               :: llx                      ! lower right longitude of flexpart grid
  real               :: lly                      ! lower right latitude of flexpart grid
  real               :: dx                       ! longitudinal resolution of flexpart grid
  real               :: dy                       ! latitudinal resolution of flexpart grid
  integer            :: nxshift                  ! longitudinal offset in grid cells relative to 180W for global output
  real, dimension(maxlev) :: outheight           ! height of model layers for grid_init files (m)
  real, dimension(:), allocatable :: gbl_lon     ! longitudes of flexpart domain
  real, dimension(:), allocatable :: gbl_lat     ! latitudes of flexpart domain

  ! time variables
  ! --------------

  real(kind=8)       :: juldatei                  ! julian start date
  real(kind=8)       :: juldatef                  ! julian end date
  integer            :: nday                      ! number of days in inversion interval
  integer            :: nyr                       ! number of years in inversion interval
  integer            :: nd_nee                    ! number of NEE values per day (in prior NEE file)
  integer            :: nt_flx                    ! number of flux values per year in prior flux files
  integer            :: nflx                      ! number of input flux time steps falling within inversion interval
  integer            :: ntstate                   ! number of time steps in state vector
  real               :: statres                   ! time resolution of the state vector (for CO2 1 day) (days) 
  real               :: statres_hr                ! sub-daily time resolution of the state vector (hours) 
  real               :: flxtres                   ! time resolution of fluxes (for GHG same as statres) (days) 
  real(kind=8), dimension(:), allocatable :: stat_time ! time dimension state vector
  real(kind=8), dimension(:), allocatable :: flx_time  ! time dimension prior fluxes
  real, dimension(:), allocatable         :: hloc      ! local time to UTC conversion

  ! grid variables
  ! --------------

  real               :: rllx                      ! lower right longitude of inversion domain
  real               :: rlly                      ! lower right latitude of inversion domain
  real               :: rurx                      ! upper right longitude of inversion domain
  real               :: rury                      ! upper right latitude of inversion domain
  real               :: rdx                       ! longitudinal resolution in inversion domain
  real               :: rdy                       ! latitudinal resolution in inversion domain
  integer            :: nxregrid                  ! number of longitudinal grid cells in inversion domain 
  integer            :: nyregrid                  ! number of latitudinal grid cells in inversion domain
  real, dimension(:), allocatable      :: reg_lon ! longitudes of inversion domain
  real, dimension(:), allocatable      :: reg_lat ! latitudes of inversion domain

  ! region variables
  ! ----------------

  integer                              :: nbox    ! number of boxes in inversion domain
  integer, dimension(:,:), allocatable :: nbox_xy ! grid operator
  real, dimension(:), allocatable      :: area_box! area of boxes
  integer                              :: ndvar   ! number of state variables in one time step
  integer                              :: ndt     ! number of state vector time steps per day (for CO2)
  integer                              :: nvar    ! number of state variables
  integer                              :: neig    ! number of retained eigenvalues of B

  ! observation variables
  ! ---------------------

  integer                              :: nrec    ! number of receptor sites or campaigns
  integer                              :: nobs    ! total number of observations
  character(recname_len), dimension(:), allocatable :: recname ! names of receptors

  ! uncertainty variables
  ! ---------------------

  real(kind=8), dimension(:), allocatable   :: evals   ! eigenvalues of B
  real(kind=8), dimension(:,:), allocatable :: evecs   ! eigenvectors of B
  real(kind=8), dimension(:,:), allocatable :: cort    ! temporal correlation matrix of B
 
  ! ouput files
  ! -----------

  integer, parameter :: logid = 500

end module mod_var


