!---------------------------------------------------------------------------------------
! FLEXINVERT: interpz
!---------------------------------------------------------------------------------------
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!
!  Copyright 2017, Rona Thompson
!---------------------------------------------------------------------------------------
!
!> interpz 
!!
!! Purpose:    Interpolates concentration data vertically to the flexpart levels
!!
!! Interface:
!!
!!    Inputs
!!             nz      -  number of vertical layers in input
!!             geoh    -  geopotential heights of layers in input
!!             gridin  -  input data
!!             gridout -  output data (zero array on input)
!!
!---------------------------------------------------------------------------------------

subroutine interpz(nz, geoh, gridin, gridout)

  use mod_var

  implicit none

  integer,                               intent(in)     :: nz
  real, dimension(nz),                   intent(in)     :: geoh
  real, dimension(nxgrid,nygrid,nz),     intent(in)     :: gridin
  real, dimension(nxgrid,nygrid,nzgrid), intent(in out) :: gridout

  integer                                               :: ix, jy, kz
  integer                                               :: z, z1, z2
  real                                                  :: ht

  do kz = 1, nzgrid
    do jy = 1, nygrid
      do ix = 1, nxgrid     
        ht = outheight(kz)
        z = minloc(abs(geoh(:)-ht),dim=1,mask=abs(geoh(:)-ht).eq.minval(abs(geoh(:)-ht)))
        if( geoh(z).lt.ht ) then
          z1 = z
          z2 = min(z+1,nz)
        else
          z1 = max(z-1,1)
          z2 = z  
        endif
        if( z1.eq.z2 ) then
          gridout(ix,jy,kz) = gridin(ix,jy,z)
        else  
          gridout(ix,jy,kz) = gridin(ix,jy,z1)+(gridin(ix,jy,z2)-gridin(ix,jy,z1)) * &
                              (ht-geoh(z1))/(geoh(z2)-geoh(z1))
        endif
      end do
    end do
  end do

end subroutine interpz


